
import subprocess
import os
from pathlib import Path
from .utils import Color
import shutil
color = Color()
cli_path = Path(os.path.dirname(__file__)).resolve()
project_path = Path(os.getcwd()).resolve()

class Initializer:
    def __init__(self):
        pass

    def create(self):
        # create global .logicbook dir
        self.create_config_dir()
        
        # create logics dir
        self.create_logics_dir()


    def create_config_dir(self):
        # create global .logicbook dir
        source_dir_path = cli_path.joinpath('../templete/.logicbook').resolve()
        target_dir_path = project_path.joinpath('.logicbook').resolve()
        print(color.green("create .logicbook dir"))
        shutil.copytree(source_dir_path, target_dir_path)

    def create_logics_dir(self):
        # create logics dir
        source_dir_path = cli_path.joinpath('../templete/logics').resolve()
        target_dir_path = project_path.joinpath('logics').resolve()
        print(color.green("create logics dir"))
        shutil.copytree(source_dir_path, target_dir_path)




import requests
from logic1 import sum_logic
from manager import Logic
from test_logic1 import TestSum

mylogic = Logic(
    name="Sum",
    func=sum_logic,
    readme="logic1.md",
)

# TODO: other name of mylogic
#TODO add example by running logic
mylogic.add_example(
    name="default", 
    args={
        "a": 1,
        "b": 2,
    })
mylogic.add_test(name="Test Class of Logic1", cls=TestSum)


import os
import inspect

def test():
    print("test")

print(os.path.abspath(inspect.getfile(test)))

class Test:
    def __init__(self) -> None:
        pass

    def test1(self):
        print("test1")

    def test2(self):
        print("test2")

import unittest
def sum_logic(a, b):
    return a + b

class TestSum(unittest.TestCase):

    def test_sum(self):
        actual = sum_logic(1, 2)
        self.assertEqual(3, actual)

def getLocalMethods(clss):
    import types
    result = [ ]
    print(clss.__dict__, types.FunctionType)
    for var in clss.__dict__:
        val = clss.__dict__[var]
        if isinstance(val, types.FunctionType) and var.startswith('test'):
            result.append(var)
    return sorted(result)
    
print(Test)
print(getLocalMethods(Test))
print(getLocalMethods(TestSum))
methods = getLocalMethods(TestSum)
for method in methods:
    print(method)
    print(TestSum.__dict__[method])
    print(TestSum.__dict__[method].__name__)


from fastapi import FastAPI
from starlette.middleware.cors import CORSMiddleware
from watchdog.events import FileSystemEventHandler
from watchdog.observers import Observer
import os
import subprocess
import sys
import time
import threading
from pydantic import BaseModel
from manager import LogicManager, Logic
import importlib
from pathlib import Path
import inspect
core_dir = Path(os.path.dirname(__file__)).resolve()
project_dir = Path(os.getcwd()).resolve()

app = FastAPI()
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,   # 追記により追加
    allow_methods=["*"],      # 追記により追加
    allow_headers=["*"]       # 追記により追加
)

app.manager = LogicManager()

@app.on_event("startup")
async def startup_event():
    print("startup")
    # run ui
    ui_thread = threading.Thread(target=run_ui)
    ui_thread.start()
    # run watcher
    watcher_thread = threading.Thread(target=run_watcher)
    watcher_thread.start()
    # add logic
    logics_dir = project_dir.joinpath('logics').resolve()
    for file in logics_dir.glob('**/*_book.py'):
        print(file.name, file.parent)
        sys.path.append(str(file.parent)) # TODO: remove pycache
        module_name = file.name.split('.')[0]
        module = importlib.import_module(module_name, file.parent)
        
        # add logic
        for name in dir(module):
            if isinstance(getattr(module, name), Logic):
                app.manager.add_logic(getattr(module, name))


class ExecuteLogicModel(BaseModel):
    id: str
    args : dict

@app.post("/execute/logic")
async def execute_logic(data: ExecuteLogicModel):
    print(data)
    result = None
    for logic in app.manager.logics:
        if logic.id == data.id:
            result = logic.func(**data.args)
    return result


class ExecuteTestModel(BaseModel):
    id: str

@app.post("/execute/test")
async def execute_test(data: ExecuteTestModel):
    for logic in app.manager.logics:
        for test in logic.tests:
            if test.id == data.id:
                test.run()
    return None

@app.get("/data")
async def get_data():
    print(len(app.manager.logics))
    print([logic.name for logic in app.manager.logics])
    data = [logic.json() for logic in app.manager.logics]
    print(data)
    return data

def run_ui():
    proc = subprocess.run(["python3", "-m", "http.server", "7000", "--directory", "build"], cwd=str(core_dir.joinpath("../ui")))

def run_watcher():
    target_dir = project_dir
    event_handler = FileChangeHandler()
    observer = Observer()
    observer.schedule(event_handler, target_dir, recursive=True)
    observer.start()
    try:
        while True:
            time.sleep(0.1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()

class FileChangeHandler(FileSystemEventHandler):
    def on_created(self, event):
        filepath = event.src_path
        filename = os.path.basename(filepath)
        print('%s created' % filename)

    def on_modified(self, event):
        filepath = event.src_path
        filename = os.path.basename(filepath)
        print('%s changed' % filename)
        for logic in app.manager.logics:
            # update logic if logic file is changed
            if str(logic.func_path) == filepath or str(logic.book_path) == filepath or str(logic.readme_path) == filepath:
                print('update logic')
                logic.update()
                break
            # run test if test file is changed
            for test in logic.tests:
                if str(test.path) == filepath:
                    print("run test", filename)
                    test.run()
                    logic.update()
                    break

    def on_deleted(self, event):
        filepath = event.src_path
        filename = os.path.basename(filepath)
        print('%s deleted' % filename)

    def on_moved(self, event):
        filepath = event.src_path
        filename = os.path.basename(filepath)
        print('%s moved' % filename)




import requests
from logic2 import join_string
from manager import Logic
from test_logic2 import TestJoin

mylogic = Logic(
    name="JoinString",
    func=join_string,
)

mylogic.add_example(
    name="default", 
    args={
        "str1": "hello",
        "str2": "world",
    }
)
mylogic.add_test(name="Test Class of Logic1", cls=TestJoin)



class Color:
    def __init__(self):
        self.RED = '\033[31m'
        self.GREEN = '\033[32m'
        self.YELLOW = '\033[33m'
        self.END = '\033[0m'
    
    def red(self, text):
        print(self.RED + text + self.END)

    def yellow(self, text):
        print(self.YELLOW + text + self.END)

    def green(self, text):
        print(self.GREEN + text + self.END)




import unittest
from logic1 import sum_logic

class TestSum(unittest.TestCase):

    def test_sum(self):
        actual = sum_logic(1, 2)
        self.assertEqual(3, actual)

    def test_sum2(self):
        actual = sum_logic(1, 2)
        self.assertEqual(3, actual)


if __name__ == '__main__':
    unittest.main()


from pydantic import BaseModel
import yaml
class Config(BaseModel):
    path: str
    
def get_config(config_path):
    with open(config_path, 'r') as yml:
        config_yaml = yaml.load(yml)
    return Config(**config_yaml)


import unittest
from logic2 import join_string

class TestJoin(unittest.TestCase):

    def test_join(self):
        actual = join_string("hello", "world")
        self.assertEqual("hello world", actual)

if __name__ == '__main__':
    unittest.main()



from typing import List
import subprocess
import uuid
from pathlib import Path
import os
import inspect
core_dir = Path(os.path.dirname(__file__)).resolve()
project_dir = Path(os.getcwd()).resolve()

class LogicManager:
    def __init__(self):
        self.logics: List[Logic] = []

    def add_logic(self, logic):
        self.logics.append(logic)

    def get_logic(self, name):
        for logic in self.logics:
            if logic.name == name:
                return logic
        return None

def get_test_names(cls):
    import types
    result = [ ]
    print(cls.__dict__, types.FunctionType)
    for var in cls.__dict__:
        val = cls.__dict__[var]
        if isinstance(val, types.FunctionType) and var.startswith('test'):
            result.append(var)
    return sorted(result)

class Logic:
    def __init__(self, name, func, readme=None) -> None:
        self.id = str(uuid.uuid4())
        self.name=name
        self.book_path = Path(os.path.abspath((inspect.stack()[1])[1]))
        self.func_path = Path(os.path.abspath(inspect.getfile(func)))
        self.readme_path = self.book_path.parent.joinpath(readme).resolve()
        self.func=func
        self.readme = self.get_markdown(self.readme_path) if self.readme_path != None else None
        self.tests=[]
        self.examples = []
        self.code = inspect.getsource(func)

    def update(self):
        self.readme = self.get_markdown(self.readme_path) if self.readme_path != None else None
        self.code = inspect.getsource(self.func)
        for test in self.tests:
            test.run()

    def get_markdown(self, path):
        md = None
        with open(path) as f:
            md = f.read()
        return md

    def get_code(self):
        path = self.func_path
        code = None
        with open(path) as f:
            code = f.read()
        return code

    def add_example(self, name, args):
        self.examples.append(Example(name, args))

    def add_test(self, name, cls):
        print(cls.__name__)
        testnames = get_test_names(cls)
        for name in testnames:
            func = cls.__dict__[name]
            cls_name = cls.__name__
            test = Test(name, cls_name, func)
            test.run()
            self.tests.append(test)

    def json(self):
        return {
            "id": self.id,
            "name": self.name,
            "book_path": str(self.book_path),
            "func_path": str(self.func_path),
            "readme_path": str(self.readme_path),
            "readme": self.readme,
            "tests": [test.json() for test in self.tests],
            "examples": [ex.json() for ex in self.examples],
            "code": self.code,
        }

class Example:
    def __init__(self, name, args):
        self.id = str(uuid.uuid4())
        self.name=name
        self.args=args

    def json(self):
        return {
            "id": self.id,
            "name": self.name,
            "args": self.args,
        }

class Test:
    def __init__(self, name, cls_name, func):
        self.id = str(uuid.uuid4())
        self.name=name
        self.cls_name=cls_name
        self.func=func
        self.path = os.path.abspath(inspect.getfile(func))
        self.status="unknown"
        self.code = inspect.getsource(func)
        self.result = None
    
    def get_code(self):
        path = self.path
        code = None
        with open(path) as f:
            code = f.read()
        return code

    def json(self):
        return {
            "id": self.id,
            "name": self.name,
            "cls_name": self.cls_name,
            "path": self.path,
            "code": self.code,
            "status": self.status,
            "result": self.result,
        }

    def run(self):
        command = ["python3", self.path, "{}.{}".format(self.cls_name, self.func.__name__)]
        proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stdout_data, stderr_data = proc.communicate()
        print(proc.returncode, stdout_data, stderr_data)
        if proc.returncode == 0:
            self.status = "success"
            self.result = stderr_data.decode('utf-8')
        else:
            self.status = "failure"
            self.result = stderr_data.decode('utf-8')


import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '../core'))

from .utils import Color
from ..core.server import app
import uvicorn
print(sys.path)
color = Color()

class Runner:
    def __init__(self):
        pass

    def run_server(self):
        print(color.yellow("Start server"))
        uvicorn.run(app, host="0.0.0.0", port=8000)